--
-- FlieglHakenLift
--
-- @author  Manuel L. (1.LS-Modschmiede)
-- @date  25/09/08

FlieglHakenLift = {};

function flieglHakenLift:load(xmlFile)

function FlieglHakenLift.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Trailer, specializations);
end;

    self.fillScale = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.fillScale#value"), 1);

	-- Defines attributes of rootArm
	local rootArmNode = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, "vehicle.rootArm#index"));
    if rootArmNode ~= nil then
        self.rootArm = {};
        self.rootArm.node = rootArmNode;
        local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.rootArm#minRot"));
        self.rootArm.minRot = {};
        self.rootArm.minRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        self.rootArm.minRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        self.rootArm.minRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));
		
        x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.rootArm#maxRot"));
        self.rootArm.maxRot = {};
        self.rootArm.maxRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        self.rootArm.maxRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        self.rootArm.maxRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        self.rootArm.rotTime = Utils.getNoNil(getXMLString(xmlFile, "vehicle.rootArm#rotTime"), 2)*1000; --ms
    end;
	
	-- Defines attributes of mainArm
	local mainArmNode = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, "vehicle.mainArm#index"));
    if mainArmNode ~= nil then
        self.mainArm = {};
        self.mainArm.node = mainArmNode;
        local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.mainArm#minRot"));
        self.mainArm.minRot = {};
        self.mainArm.minRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        self.mainArm.minRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        self.mainArm.minRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));
		
        x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.mainArm#maxRot"));
        self.mainArm.maxRot = {};
        self.mainArm.maxRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        self.mainArm.maxRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        self.mainArm.maxRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        self.mainArm.rotTime = Utils.getNoNil(getXMLString(xmlFile, "vehicle.mainArm#rotTime"), 2)*1000; --ms	
		
		-- Defines a fixpoint to align the hydraulic with this point
		self.mainArm.hydraulicPoint = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, "vehicle.mainArm#hydraulicPoint"));
    end;
	
	-- Defines attributes of workingArm
	local workingArmNode = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, "vehicle.workingArm#index"));
	if workingArmNode ~= nil then
		self.workingArm = {};
		self.workingArm.node = workingArmNode;
		
		x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.workingArm#maxTrans"));
		self.workingArm.minHeight = {};
		self.workingArm.minHeight[1] = Utils.degToRad(Utils.getNoNil(x, 0));
		self.workingArm.minHeight[2] = Utils.degToRad(Utils.getNoNil(y, 0));
		self.workingArm.minHeight[3] = Utils.degToRad(Utils.getNoNil(z, 0));
		
		local x1, y1, z1 = getTranslation(self.workingArm.node);
		self.workingArm.maxHeight = {};
		self.workingArm.maxHeight[1] = x1;
		self.workingArm.maxHeight[2] = y1;
		self.workingArm.maxHeight[3] = z1;
		
		self.workingArm.transTime = Utils.getNoNil(getXMLString(xmlFile, "vehicle.workingArm#rotTime"), 2)*1000; --ms
	end;
	
	-- Defines attributes of objectLock
	local objectLockNode = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, "vehicle.objectLock#index"));
    if objectLockNode ~= nil then
        self.objectLock = {};
        self.objectLock.node = objectLockNode;
        local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.objectLock#minRot"));
        self.objectLock.minRot = {};
        self.objectLock.minRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        self.objectLock.minRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        self.objectLock.minRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));
		
        x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.objectLock#maxRot"));
        self.objectLock.maxRot = {};
        self.objectLock.maxRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        self.objectLock.maxRot[2] = self.objectLock.minRot[2];
        self.objectLock.maxRot[3] = self.objectLock.minRot[3];
		
        self.objectLock.rotTime = 100; --ms
    end;
	
	-- Defines attributes of supportarm/wheel
	local supportNode = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, "vehicle.support#index"));
	if supportNode ~= nil then
		self.support = {};
		self.support.node = supportNode;
		
		local x1, y1, z1 = getTranslation(self.support.node);
		self.support.minHeight = {};
		self.support.minHeight[1] = x1;
		self.support.minHeight[2] = y1;
		self.support.minHeight[3] = z1;
		
		x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.support#maxY"));
		self.support.maxHeight = {};
		self.support.maxHeight[1] = self.support.minHeight[1]
		self.support.maxHeight[2] = Utils.degToRad(Utils.getNoNil(y, 0));
		self.support.maxHeight[3] = self.support.minHeight[3]
	end;
	
	
	-- Defines attributes of hydraulic
	self.hydraulic = {};
	self.hydraulic.node = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, "vehicle.hydraulic#indexHydraulic"));
	self.hydraulic.punch = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, "vehicle.hydraulic#indexPunch"));
	self.hydraulic.translationPunch = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, "vehicle.hydraulic#translationPunch"));
	local ax, ay, az = getWorldTranslation(self.hydraulic.punch);
    local bx, by, bz = getWorldTranslation(self.hydraulic.translationPunch);
    self.hydraulic.punchDistance = Utils.vector3Length(ax-bx, ay-by, az-bz);
	
	-- Defines trailingtruck
	self.trailingTruckBackAngle = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.trailingTruck#backAngle"), 25);
	self.trailingTruckActive = true;	
	
	-- Defines object attaching point
	local objectAttacher = {};
	objectAttacher.joint = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, "vehicle.objectAttacher#index"));
	objectAttacher.index = 0;
	self.object = objectAttacher;
	self.attachedObject = nil;
	self.objectAttachable = false;
	self.objectInRange = nil;
	
	-- Defines some attributes to get a floating loading action because GiantsEngine currently don't support collisions between 2 joined Objects
	self.maxRotLimit = Utils.degToRad(Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.rotationMax#xRot"),0));
	self.changingAngle = Utils.degToRad(-64.215);
	self.nullAngle = Utils.degToRad(-60);	
	
	self.keys = {};
	local i=0;
    while true do
        local baseName = string.format("vehicle.inputs.input(%d)", i);
        local inputName = getXMLString(xmlFile, baseName.. "#name");
        if inputName == nil then
            break;
        end;
        local inputKey = getXMLString(xmlFile, baseName.. "#key");
        if Input[inputKey] == nil then
            print("Error: invalid key '" .. inputKey .. "'  for input event '" .. inputName .. "'");
            break;
        end;
        self.keys[inputName] = Input[inputKey];
        i = i+1;
    end;
	
    delete(xmlFile);
	
	-- Defines the arm's moving directness
	self.armMoving = 0;
	-- Defines if the mainarm is locked. If true, container can be dumped. If false, the container can be unloaded 
	self.isArmLocked = false;
	-- Defines the point where you can switch between locked and unlocked arm state
	self.isChangeable = false;
	-- Defines the point where yu can attach/detach an object
	self.attachable = false;
	
	-- defines this trailer as an hakenlift trailer
	self.isHakenlift = true;
	
	self.runOnlyOnce = true;
	
	
	self.hudExtendedHelpPosX = 0.38;
	self.hudExtendedHelpPosY = 0.892; --0.878
	self.hudExtendedHelpWidth = 0.220;    
    self.hudExtendedHelpHeight = 0.10;
	self.hudTextPosX = 0.38;
	self.hudTextPosY = 0.916;
    self.hudHakenliftOverlay = Overlay:new("hudHakenliftMenu", "hud_Help_base.png", self.hudExtendedHelpPosX, self.hudExtendedHelpPosY, self.hudExtendedHelpWidth, self.hudExtendedHelpHeight);

	setUserAttribute(self.fillRootNode, "vehicleType", "Integer", 0);
	
    return self;
end;

function FlieglHakenLift:delete()
	if self.hudHakenliftOverlay ~= nil then
		self.hudHakenliftOverlay:delete();
	end;
    Trailer.delete(self);
end;

function FlieglHakenLift:mouseEvent(posX, posY, isDown, isUp, button)
    Trailer.mouseEvent(self, posX, posY, isDown, isUp, button);
end;

function FlieglHakenLift:keyEvent(unicode, sym, modifier, isDown)
    Trailer.keyEvent(self, unicode, sym, modifier, isDown);
	
	if self.attachedObject ~= nil then
        self.attachedObject:keyEvent(unicode, sym, modifier, isDown)
    end;
	
	if self.attacherVehicle ~= nil then		
		local localObject = self.object;
		
		if isDown then
		    if sym == self.keys.ATTACH_OBJECT and self.attachable then
		        if self.attachedObject ~= nil then
					if self.objectLock ~= nil then
						local rot = {self.objectLock.minRot[1], self.objectLock.minRot[2], self.objectLock.minRot[3]};
						setRotation(self.objectLock.node, unpack(rot));
					end;
					self:delCamera();
		            self.attachedObject:onDetach();
					removeJoint(localObject.index);
		            localObject.index = 0;
		            self.attachedObject = nil;
		        else
		            if self.objectAttachable then
			            if self.objectInRange ~= nil then
			                self.attachedObject = self.objectInRange;
							self.attachedObject:onAttach(self);
			                local constr = JointConstructor:new();
			                constr:setActors(self.rootNode, self.objectInRange.rootNode);
			                constr:setJointTransforms(localObject.joint, self.objectInRange.attacherJoint);
							constr:setRotationLimit(0, Utils.degToRad(-179), Utils.degToRad(179));
							constr:setRotationLimit(1, Utils.degToRad(-50), Utils.degToRad(50));--0
							constr:setRotationLimit(2, Utils.degToRad(-179), Utils.degToRad(179));
			                for i=1, 3 do		                    
			                    constr:setTranslationLimit(i-1, true, 0, 0);
			                end;
			                localObject.index = constr:finalize();
							self:addCamera();
							if self.objectLock ~= nil then
								local rot = {self.objectLock.maxRot[1], self.objectLock.maxRot[2], self.objectLock.maxRot[3]};
								setRotation(self.objectLock.node, unpack(rot));
							end;
			            end;
					end;
		        end;
			else if sym == Input.KEY_k then
				self.trailingTruckActive = not self.trailingTruckActive;
			else if sym == Input.KEY_x and self.isChangeAble then
				self.isArmLocked = not self.isArmLocked;
			end;
		end;
	end;
end;

function FlieglHakenLift:update(dt)

	if self.runOnlyOnce then
		local firstObject = self:ObjectInRange(true);
		if firstObject ~= nil then
			local localObject = self.object;
			self.objectAttachable = true;
			self.attachedObject = firstObject;
			self.attachedObject:onAttach(self);
			local constr = JointConstructor:new();
			constr:setActors(self.rootNode, firstObject.rootNode);
			constr:setJointTransforms(localObject.joint, firstObject.attacherJoint);
			for i=1, 3 do		                    
				constr:setTranslationLimit(i-1, true, 0, 0);
				constr:setRotationLimit(i-1, 0, 0);
			end;
			localObject.index = constr:finalize();
			if self.objectLock ~= nil then
				local rot = {self.objectLock.maxRot[1], self.objectLock.maxRot[2], self.objectLock.maxRot[3]};
				setRotation(self.objectLock.node, unpack(rot));
			end;
		end;
		self.runOnlyOnce = false;
	end;

	self.armMoving = 0;
	self.isChangeAble = false;
	self.attachable = false;
	-- Check if mainArm is in the right postion for changing arm lockstate or if arm is in right position to attach/detach an object
	if self.mainArm ~= nil and self.rootArm ~= nil then
		local xMain, yMain, zMain = getRotation(self.mainArm.node);
		local xRoot, yRoot, zRoot = getRotation(self.rootArm.node);
		local xArm, yArm, zArm = getTranslation(self.workingArm.node);
		if xMain > Utils.degToRad(-0.005) and xRoot > Utils.degToRad(-0.005) and zArm > 0.73 then
			self.isChangeAble = true;
		end;
		if xMain < Utils.degToRad(-143) then
			self.attachable = true;
		end;
	end;	
	
	local angle = 0;
	local backAngle = 0;	
	-- Calucaltion for trailingtruck
	if self.trailingTruckActive and self.attacherVehicle ~= nil then
		local x1,y1,z1 = localDirectionToWorld(self.attacherVehicle.rootNode, 0, 0, 1);
		local x2,y2,z2 = localDirectionToWorld(self.rootNode, 0, 0, 1);
		--2. skalar produkt, wobei y wert ignorieren
		local dot = x1*x2 + z1*z2;
		--3. durch laengen teilen
		dot = dot / (Utils.vector2Length(x1,z1) * Utils.vector2Length(x2,z2));
		--4. arccosine nehmen
		angle = math.acos(dot);		
		-- y component of cross product
		local cy = z1 * x2 - x1 * z2;		
		if cy < 0 then
			angle = -angle;
		end;		
		backAngle  = self.trailingTruckBackAngle * (angle / 50);				
	end;			
	-- update of wheels
	for i=1, table.getn(self.wheels) do
		local x,y,z = getRotation(self.wheels[i].repr);			
		local newX, newY, newZ = getWheelShapePosition(self.wheels[i].node, self.wheels[i].wheelShape);
		setTranslation(self.wheels[i].repr, newX, newY, newZ);
		local axleSpeed = getWheelShapeAxleSpeed(self.wheels[i].node, self.wheels[i].wheelShape)*3.14159/180;
		x = x+axleSpeed*dt/1000.0;
		if i == 3 or i == 4 then
			setWheelShapeProps(self.wheels[i].node, self.wheels[i].wheelShape, 0, 0, backAngle);
			setRotation(self.wheels[i].repr, x, backAngle, z);					
		else
			steeringAngle = 0;
			setRotation(self.wheels[i].repr, x, steeringAngle, z);
		end;
	end;
	
	
	if self.attacherVehicle ~= nil and g_currentMission.currentVehicle ~= nil and g_currentMission.currentVehicle == self.attacherVehicle then
		
		if self.attachedObject == nil or (self.attachedObject ~= nil and (self.attachedObject.isActive == nil or (self.attachedObject.isActive ~= nil and not self.attachedObject.isActive))) then
			if Input.isKeyPressed(self.keys.LOWER_ARM) then
				self.armMoving = 1;
			elseif Input.isKeyPressed(self.keys.LIFT_ARM) then
				self.armMoving = -1;
			end;
		end;
	
	
		if self.attachedObject == nil then
			self.objectInRange = self:ObjectInRange(false);
			if self.objectInRange ~= nil then
				self.objectAttachable = true;
			else
				self.objectAttachable = false;
			end;
		else
			if self.object ~= nil and (self.armMoving ~= 0 or self.workingArmMoving ~= 0) then
				local localObject = self.object;				
				if self.isArmLocked then
					-- Arm is locked for dumping
					setJointRotationLimit(localObject.index, 0, true, Utils.degToRad(0), Utils.degToRad(0));
					setJointRotationLimit(localObject.index, 1, true, Utils.degToRad(0), Utils.degToRad(0));
					setJointRotationLimit(localObject.index, 2, true, Utils.degToRad(0), Utils.degToRad(0));
				else
					-- This calculates current joint angleRegulation
					local x, y, z = getRotation(self.mainArm.node);
					local angleRelation = ((x / self.changingAngle) * self.maxRotLimit);
					local ax, ay, az = getRotation(localObject.joint);
					setRotation(localObject.joint, ax, ay, angleRelation);

					if x < self.changingAngle then
						-- This calculates the current rotationlimit depending on the current mainArm rotation
						local angle = (x - self.changingAngle)/(self.mainArm.maxRot[1] - self.changingAngle)*Utils.degToRad(50);
						setJointRotationLimit(localObject.index, 0, true, Utils.degToRad(-179), Utils.degToRad(179));
						setJointRotationLimit(localObject.index, 1, true, angle*-1, angle);
						setJointRotationLimit(localObject.index, 2, true, Utils.degToRad(-179), Utils.degToRad(80));
					elseif x >= self.changingAngle and x < self.nullAngle then
						-- This calculates the current rotationlimit depending on the current mainArm rotation (required for a floating loading action)
						local angle = (x - self.nullAngle)/(self.changingAngle - self.nullAngle)*Utils.degToRad(5);
						setJointRotationLimit(localObject.index, 2, true, Utils.degToRad(0), angle);					
					else
						setJointRotationLimit(localObject.index, 2, true, Utils.degToRad(0), Utils.degToRad(0));
						setJointRotationLimit(localObject.index, 0, true, Utils.degToRad(0), Utils.degToRad(0));
						setJointRotationLimit(localObject.index, 1, true, Utils.degToRad(0), Utils.degToRad(0)); --neu
					end;			
				end;
				-- update joint position and rotation
	            setJointFrame(localObject.index, 0, localObject.joint);
			end;
		end;
		if self.tipState == self.TIPSTATE_CLOSED then
			if self.isArmLocked then
				if self.rootArm ~= nil and self.armMoving ~= 0 then
					local x, y, z = getRotation(self.rootArm.node);
					local rot = {x,y,z};
					local newRot = Utils.getMovedLimitedValues(rot, self.rootArm.maxRot, self.rootArm.minRot, 3, self.rootArm.rotTime, dt, self.armMoving == 1);
					setRotation(self.rootArm.node, unpack(newRot));
				end;	
			else
				local x, y, z = getRotation(self.mainArm.node);
				local ax, ay, az = getTranslation(self.workingArm.node);
				if self.mainArm ~= nil and self.armMoving ~= 0 and az < self.workingArm.minHeight[3] + 0.001 then
					local x, y, z = getRotation(self.mainArm.node);
					local rot = {x,y,z};
					local newRot = Utils.getMovedLimitedValues(rot, self.mainArm.maxRot, self.mainArm.minRot, 3, self.mainArm.rotTime, dt, self.armMoving == 1);
					setRotation(self.mainArm.node, unpack(newRot));
				end;	
				if self.workingArm ~= nil and self.armMoving ~= 0 and x > Utils.degToRad(-0.005) then
					local x, y, z = getTranslation(self.workingArm.node);
					local trans = {x,y,z};
					local newTrans = Utils.getMovedLimitedValues(trans, self.workingArm.maxHeight, self.workingArm.minHeight, 3, self.workingArm.transTime, dt, self.armMoving == -1);
					setTranslation(self.workingArm.node, unpack(newTrans));
				end;			
			end;
		else
			if self.attachedObject ~= nil then
				if self.rootArm ~= nil then
					local x, y, z = getRotation(self.rootArm.node);
					local rot = {x,y,z};
					local newRot = Utils.getMovedLimitedValues(rot, self.rootArm.maxRot, self.rootArm.minRot, 3, self.rootArm.rotTime, dt, self.tipState == self.TIPSTATE_CLOSING);
					setRotation(self.rootArm.node, unpack(newRot));
				end;
			end;
		end;
	end;
	
	-- oriented the hydraulic and the punch towards a fixpoint on the mainArm
	if self.hydraulic ~= nil and self.mainArm ~= nil then
		local ax, ay, az = getWorldTranslation(self.hydraulic.node);
		local bx, by, bz = getWorldTranslation(self.mainArm.hydraulicPoint);
		local x, y, z = worldDirectionToLocal(getParent(self.hydraulic.node), bx-ax, by-ay, bz-az);
		setDirection(self.hydraulic.node, x*-1, y*-1, z*-1, 0, 1, 0);
		if self.hydraulic.punch ~= nil then
			local distance = Utils.vector3Length(ax-bx, ay-by, az-bz);
			setTranslation(self.hydraulic.punch, 0, 0, (distance-self.hydraulic.punchDistance)*-1);
		end;
	end;

	-- sets trailer attributes depending on an attached object
	if self.attachedObject ~= nil then
		self.tipState = self.attachedObject.tipState;
		self.lastFillDelta = self.attachedObject.lastFillDelta;
		self.fillLevel = self.attachedObject.fillLevel;
		self.capacity = self.attachedObject.capacity;
		self.fillTypes[self.FILLTYPE_WHEAT] = self.attachedObject.fillTypes[self.FILLTYPE_WHEAT];
		self.fillTypes[self.FILLTYPE_GRASS] = self.attachedObject.fillTypes[self.FILLTYPE_GRASS];
		if self.isArmLocked then
			self.currentFillType = self.attachedObject.realFillType;
		else
			self.currentFillType = self.FILLTYPE_UNKNOWN;
		end;
	else
		self.tipState = self.TIPSTATE_CLOSED;
		self.fillLevel = 0;
		self.capacity = 1;	
		self.fillTypes[self.FILLTYPE_WHEAT] = false;
		self.fillTypes[self.FILLTYPE_GRASS] = false;
		self.currentFillType = self.FILLTYPE_UNKNOWN;
	end;	
	
	-- set current mass
	if self.firstTimeRun then
        if self.emptyMass == nil then
            self.emptyMass = getMass(self.fillRootNode);
            self.currentMass = self.emptyMass;
        end;
        local massScale = self.massScale*self.capacity;
        local newMass = self.emptyMass + self.fillLevel*massScale;
        if newMass ~= self.currentMass then
            setMass(self.fillRootNode, newMass);
            self.currentMass = newMass;
            if self.centerOfMass ~= nil then
                setCenterOfMass(self.fillRootNode, self.centerOfMass[1], self.centerOfMass[2], self.centerOfMass[3]);
            end;
        end;
    end;

    self.firstTimeRun = true;
end;

function FlieglHakenLift:draw()
    Trailer.draw(self);
	if self.attacherVehicle ~= nil then
		if self.attachedObject ~= nil then
			self.attachedObject:draw();
		end;
		setTextBold(false);
		setTextColor(1.0, 1.0, 1.0, 1.0);
		if self.hudHakenliftOverlay ~= nil then
			self.hudHakenliftOverlay:render();
		end;	
		if g_languageShort ~= nil and g_languageShort == "de" then
			renderText(self.hudTextPosX+0.003, self.hudTextPosY+0.057, 0.018, "Tasten "..self:getCorrectKey(self.keys.LOWER_ARM).."/"..self:getCorrectKey(self.keys.LIFT_ARM)..": Arm ein-/ausfahren");	
		else
			renderText(self.hudTextPosX+0.003, self.hudTextPosY+0.057, 0.018, "Keys "..self:getCorrectKey(self.keys.LOWER_ARM).."/"..self:getCorrectKey(self.keys.LIFT_ARM)..": Run/extend arm");
		end;
		if self.trailingTruckActive then
			if g_languageShort ~= nil and g_languageShort == "de" then
				renderText(self.hudTextPosX+0.003, self.hudTextPosY+0.057-0.02, 0.018, "Tasten "..self:getCorrectKey(self.keys.ACTIVATE_TRAILINGTRUCK)..": Nachlaufachse ausschalten");
			else
				renderText(self.hudTextPosX+0.003, self.hudTextPosY+0.057-0.02, 0.018, "Key "..self:getCorrectKey(self.keys.ACTIVATE_TRAILINGTRUCK)..": Turn trailingtruck off");
			end;
		else
			if g_languageShort ~= nil and g_languageShort == "de" then
				renderText(self.hudTextPosX+0.003, self.hudTextPosY+0.057-0.02, 0.018, "Tasten "..self:getCorrectKey(self.keys.ACTIVATE_TRAILINGTRUCK)..": Nachlaufachse einschalten");
			else
				renderText(self.hudTextPosX+0.003, self.hudTextPosY+0.057-0.02, 0.018, "Key "..self:getCorrectKey(self.keys.ACTIVATE_TRAILINGTRUCK)..": Turn trailingtruck on");
			end;
		end;
		if self.isChangeAble then
			if self.isArmLocked then
				if g_languageShort ~= nil and g_languageShort == "de" then
					renderText(self.hudTextPosX+0.003, self.hudTextPosY+0.057-0.04, 0.018, "Taste "..self:getCorrectKey(self.keys.LOCK_ARM)..": Armsperre auschalten");
				else
					renderText(self.hudTextPosX+0.003, self.hudTextPosY+0.057-0.04, 0.018, "Key "..self:getCorrectKey(self.keys.LOCK_ARM)..": Turn Arm-Lock off");
				end
			else	
				if g_languageShort ~= nil and g_languageShort == "de" then
					renderText(self.hudTextPosX+0.003, self.hudTextPosY+0.057-0.04, 0.018, "Taste "..self:getCorrectKey(self.keys.LOCK_ARM)..": Armsperre einschalten");
				else
					renderText(self.hudTextPosX+0.003, self.hudTextPosY+0.057-0.04, 0.018, "Key "..self:getCorrectKey(self.keys.LOCK_ARM)..": Turn Arm-Lock on");
				end;
			end;
		end;
		if self.attachable and (self.objectInRange ~= nil or self.attachedObject ~= nil) then
			if self.attachedObject ~= nil then
				if g_languageShort ~= nil and g_languageShort == "de" then
					renderText(self.hudTextPosX+0.003, self.hudTextPosY+0.057-0.06, 0.018, "Taste "..self:getCorrectKey(self.keys.ATTACH_OBJECT)..": " .. self.attachedObject.name .. " abhngen");
				else
					renderText(self.hudTextPosX+0.003, self.hudTextPosY+0.057-0.06, 0.018, "Key "..self:getCorrectKey(self.keys.ATTACH_OBJECT)..": Detach " .. self.attachedObject.name);
				end;
			else
				if self.objectInRange ~= nil then
					if g_languageShort ~= nil and g_languageShort == "de" then
						renderText(self.hudTextPosX+0.003, self.hudTextPosY+0.057-0.06, 0.018, "Taste "..self:getCorrectKey(self.keys.ATTACH_OBJECT)..": " .. self.objectInRange.name .. " anhngen");
					else
						renderText(self.hudTextPosX+0.003, self.hudTextPosY+0.057-0.06, 0.018, "Key "..self:getCorrectKey(self.keys.ATTACH_OBJECT)..": Attach " .. self.objectInRange.name);
					end;
				end;
			end;
		else
			if self.attachedObject ~= nil then
				if g_languageShort ~= nil and g_languageShort == "de" then
					renderText(self.hudTextPosX+0.003, self.hudTextPosY+0.057-0.06, 0.018, "Sperren sie den Arm, \num am Silo auszukippen!");
				else
					renderText(self.hudTextPosX+0.003, self.hudTextPosY+0.057-0.06, 0.018, "Lock the arm to dump \nthe grain at the silo!");
				end;
			else
				if g_languageShort ~= nil and g_languageShort == "de" then
					renderText(self.hudTextPosX+0.003, self.hudTextPosY+0.057-0.06, 0.018, "Fahren Sie den Arm ganz aus, \num einen Container anzuhngen!");
				else
					renderText(self.hudTextPosX+0.003, self.hudTextPosY+0.057-0.06, 0.018, "Extend the arm completely \nto attach a container!");
				end;
			end;
		end;
	end;
end;

function FlieglHakenLift:onAttach(attacherVehicle)
	-- support wheel 
	if self.support ~= nil then
		setTranslation(self.support.node, unpack({self.support.maxHeight[1], self.support.maxHeight[2], self.support.maxHeight[3]}));
	end;
	Trailer.onAttach(self, attacherVehicle);
	if self.attachedObject ~= nil and self.attachedObject.camera ~= nil then
		self:addCamera();
	end;
end;

function FlieglHakenLift:onDetach()
	-- support wheel
	if self.support ~= nil then
		setTranslation(self.support.node, unpack({self.support.minHeight[1], self.support.minHeight[2], self.support.minHeight[3]}));
	end;
	if self.attachedObject ~= nil and self.attachedObject.camera ~= nil then
		self:delCamera();
	end;
    Trailer.onDetach(self);

end;

function FlieglHakenLift:setFillLevel(fillLevel, fillType)
	if self.attachedObject ~= nil then
		self.attachedObject:setFillLevel(fillLevel, fillType);
	end;
end;

function FlieglHakenLift:toggleTipState()
	if self.isArmLocked and self.isChangeAble and self.attachedObject ~= nil then
		self.attachedObject.toggleTipState();
		if self.attachedObject.tipState == 0 then
	        self.attachedObject:onStartTip();
	    else
	        self.attachedObject:onEndTip();
	    end;
	end;
end;

function FlieglHakenLift:ObjectInRange(extended)
    local nearestObject = nil;
	local objectCopy = self.object;
	if not extended and self.attacherVehicle == nil then
		return nearestObject;	
	else    
        local nearestDistance = 1.0;
		if extended then
			nearestDistance = 2.0;
		end;
		local px, py, pz = getWorldTranslation(objectCopy.joint);

		for i=1, table.getn(g_currentMission.trailers) do
			if g_currentMission.trailers[i] ~= self  and g_currentMission.trailers[i].attacherVehicle == nil then
				local vx, vy, vz = getWorldTranslation(g_currentMission.trailers[i].attacherJoint);
				local distance = Utils.vector3Length(px-vx, py-vy, pz-vz);
				if distance < nearestDistance then
					nearestObject = g_currentMission.trailers[i];
					nearestDistance = distance;
				end;
			end;
		end;
		if nearestObject ~= nil then
			-- Check if nearest object is a hakenlift attachable 
			if nearestObject.hakenliftObject == nil then
				nearestObject = nil;
			end;
		end;
    end;
    return nearestObject;
end;

function FlieglHakenLift:getCorrectKey(key)
	if key < 256 then
		return string.upper(string.char(key));
	elseif key > 255 and key < 266 then
		return string.upper("KP"..string.char(key-208));
	else
		return "n/a";
	end;
end;

function FlieglHakenLift:addCamera()
	if self.attacherVehicle ~= nil and self.attacherVehicle.cameras ~= nil and self.attachedObject ~= nil and self.attachedObject.camera ~= nil then
		table.insert(self.attacherVehicle.cameras, self.attachedObject.camera);
		self.attacherVehicle.numCameras = self.attacherVehicle.numCameras + 1;
	end;
end;

function FlieglHakenLift:delCamera()
	if self.attacherVehicle ~= nil and self.attacherVehicle.cameras ~= nil and self.attachedObject ~= nil and self.attachedObject.camera ~= nil then
		for i=1, table.getn(self.attacherVehicle.cameras) do
			if self.attacherVehicle.cameras[i] == self.attachedObject.camera then
				table.remove(self.attacherVehicle.cameras, i);
				self.attacherVehicle.numCameras = self.attacherVehicle.numCameras - 1;
			end;		
		end;
	end;
end;
